#!/bin/bash

# Script to automagically update Nginx reverse proxy configuration
# Artifactory login
: ${ART_LOGIN:=_internal}
# Artifactory password
: ${ART_PASSWORD:=b6a50c8a15ece8753e37cbe5700bf84f}
# Interval in seconds to check for new configuration on artifactory
CHECK_INTERVAL=10
# Path to nginx conf inside the docker image
NGINX_CONF=/etc/nginx/conf.d/artifactory.conf

function curlAuth {
	if [ -z "$ART_API_KEY" ]; then
		echo "curl -u$ART_LOGIN:$ART_PASSWORD"
	else
		echo "curl -H \"X-JFrog-Art-Api:$ART_API_KEY\""
	fi
}

function updateNginxConfIfNeeded {
	local reverseProxyConf=$(getReverseProxySnippet)
	if [ "$reverseProxyConf" != "ERROR" ] && [ ! -z "$reverseProxyConf" ]; then
		local diffWithCurrentConf=$(diff $NGINX_CONF <(echo "$reverseProxyConf"))
		if [ -n "$diffWithCurrentConf" ]
		then
			echo "ART CONFIG CHANGED : $diffWithCurrentConf" 
			echo "UPDATING NGINX CONF at  $NGINX_CONF"
			local savedConf=$(cat $NGINX_CONF)
			echo "$reverseProxyConf" > "$NGINX_CONF"

			/etc/init.d/nginx reload
			if [ $? -ne 0 ]; then
				logError "Something went wrong after loading new config, restoring the previous conf"
				echo "$savedConf" > "$NGINX_CONF"
			fi
		fi
	fi
}

function getReverseProxyConfFromPrimaryNode {
	local curl=$(curlAuth)
	local arguments=" $ART_BASE_URL/api/system/configuration/webServer"
	eval $curl $arguments
}

function getReverseProxySnippet {
	local result=$(getReverseProxySnippetFrom $ART_BASE_URL)
	echo "$result"
}

function getReverseProxySnippetFrom {
	local baseUrl=$1
	local response
	# Yes we separate declaration and assignment, if not, $? will not be properly set to the result of the curl
	local curl=$(curlAuth)
	local arguments=" --show-error --silent --fail $baseUrl/api/system/configuration/reverseProxy/nginx"
	response=$((eval $curl $arguments) 2>&1)
	local responseStatus=$?
	if [ $responseStatus -ne 0 ]; then
		logError "Couldn't retrieve the reverse proxy conf from $baseUrl, response from server $response "
		echo "ERROR"
	else
		echo "$response"
	fi
}

function waitForPrimaryNode {
	echo "[NGINX] WAITING FOR PRIMARY NODE AT $ART_BASE_URL"
	local curl=$(curlAuth)
	local arguments=" --output /dev/null --silent --head --fail $ART_BASE_URL/api/system/ping"
	until $(eval $curl $arguments)
	do
		echo "."
		sleep $CHECK_INTERVAL
	done
	echo "[NGINX] PRIMARY NODE IS UP !"
}

function logError {
	>&2 echo "$1"
}

# See how we can deduce where the PRIMARY can be reached
# One option can be : do a plain SQL request... to the artifactory_servers...
# Or better try to ask the MESOS DNS for that, so :
# If the PRIMARY_BASE_URL is specified we'll use that, if not we will try to resolve it
# from mesos DNS (if we want to make it generic we need to pass a variable to specify a DNS)
# That also mean the primary must be launched before nginx
function setPrimaryBaseUrl {
	if [ -z "$ART_PRIMARY_BASE_URL" ]; then
	    ART_INSTANCE_PORT=$(dig srv _instance._$ART_SERVICE_NAME._tcp.marathon.mesos +short | head -1 | awk '{print $3}')
    	ART_BASE_URL=http://$ART_SERVICE_NAME.marathon.mesos:$ART_INSTANCE_PORT/artifactory
    else
        ART_BASE_URL=$ART_PRIMARY_BASE_URL
    	ART_INSTANCE_PORT=$(echo $ART_BASE_URL | awk -F/ '{print $3}' | cut -d: -f2)
	fi
}


function checkAndMoveCert {
    if [ -f $MESOS_SANDBOX/artifactory.crt ] &&  [ -f $MESOS_SANDBOX/artifactory.key ]; then
        mkdir /etc/nginx/ssl
        echo "Moving SSL cert to /etc/nginx/ssl"
        mv $MESOS_SANDBOX/artifactory.crt /etc/nginx/ssl/artifactory.crt
        mv $MESOS_SANDBOX/artifactory.key /etc/nginx/ssl/artifactory.key
    fi
}

# Set primary host
setPrimaryBaseUrl
waitForPrimaryNode
checkAndMoveCert

# Then we check every n seconds for a diff between file conf and the one we get from artifactory
while [ true ]
do
    setPrimaryBaseUrl
	updateNginxConfIfNeeded
	sleep $CHECK_INTERVAL
done
